/*
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 *
 * Contributor(s):
 *
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package org.glassfish.scripting.jruby;

import org.glassfish.api.admin.ParameterNames;
import org.glassfish.api.admin.ServerEnvironment;
import org.glassfish.api.container.EndpointRegistrationException;
import org.glassfish.api.container.RequestDispatcher;
import org.glassfish.api.deployment.DeployCommandParameters;
import org.glassfish.api.deployment.Deployer;
import org.glassfish.api.deployment.DeploymentContext;
import org.glassfish.api.deployment.MetaData;
import org.glassfish.scripting.jruby.common.config.JRubyConfig;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Service;

import java.io.IOException;
import java.util.Properties;
import java.util.logging.Level;

/**
 * Deployer for Rails applications
 *
 * @author Jerome Dochez
 * @author Vivek Pandey
 */
@Service
public class JRubyDeployer implements Deployer<JRubyContainer, JRubyApplication> {

    @Inject
    ServerEnvironment env;

    @Inject
    RequestDispatcher dispatcher;

    @Inject
    org.glassfish.scripting.jruby.config.JrubyContainer config;

    private String appName;


    public boolean prepare(DeploymentContext context) {
        return true;
    }

    /**
     * Loads the meta date associated with the application.
     *
     * @parameters type type of metadata that this deployer has declared providing.
     */
    public <T> T loadMetaData(Class<T> type, DeploymentContext context) {
        return null;
    }

    /**
     * Returns the meta data assocated with this Deployer
     *
     * @return the meta data for this Deployer
     */
    public MetaData getMetaData() {
        return new MetaData(false, null, null);
    }

    public JRubyApplication load(JRubyContainer container, DeploymentContext context) {
        this.appName = context.getCommandParameters(DeployCommandParameters.class).name;
        String railsRoot;

        try {
            railsRoot = context.getSourceDir().getCanonicalPath();
        } catch (IOException e) {
            railsRoot = context.getSourceDir().getAbsolutePath();
        }

        String contextRoot = getContextRoot(context, env.getStartupContext().getArguments());

        JRubyConfig jrubyConfig = new JRubyConfigImpl(context.getModuleProps(), context.getCommandParameters(DeployCommandParameters.class).name, railsRoot, contextRoot, context.getLogger(), new JRubyMonitorListnerImpl(), config);
        try {
            return new JRubyApplication(context, env, dispatcher, jrubyConfig);
        } catch (IOException e) {
            context.getLogger().log(Level.SEVERE, e.getMessage(), e);
            return null;
        }
    }

    public void unload(JRubyApplication container, DeploymentContext context) {
        try {
            //Fix for NPE, if load failed, the container is null, why is v3 trying to unload if there is
            //no container?
            if(container != null)
                dispatcher.unregisterEndpoint(container.getContextRoot(), container);
            container = null;
            context.getLogger().info(Messages.format(Messages.INFO_UNLOADING_APP,appName));
        } catch (EndpointRegistrationException e) {
            context.getLogger().log(Level.SEVERE,
                    Messages.format(Messages.ERR_UNLOAD_APP, container.getContextRoot()), e);
        }
    }

    public void clean(DeploymentContext context) {
    }

    private static final String CONTEXT_ROOT_ARG ="--"+ ParameterNames.CONTEXT_ROOT;
    private String getContextRoot(DeploymentContext context, Properties props) {
        String contextRoot = props.getProperty(CONTEXT_ROOT_ARG);
        DeployCommandParameters params = context.getCommandParameters(DeployCommandParameters.class);
        //For gem we get contextroot as part of startup context and in normal server deploy it comes from
        //DeploymentContext properties. We should override contextroot from startupcontext by the one
        // from DeploymentContext
        if(params.contextroot != null){
            contextRoot = params.contextroot;
        }
        if (contextRoot == null || contextRoot.length() == 0) {
            contextRoot = "/" + params.name;
        }

        //incase someone deployed the application with --contextroot without no beginning '/'
        if(contextRoot.charAt(0) != '/')
            contextRoot = "/" + contextRoot;
        return contextRoot;
    }
    
}

